package bonephone;

import org.mbus.base.*;
import org.mbus.layer1.*;
import org.mbus.layer2.*;
import java.util.*;


/** Class to send multiple commands via MBUS to a dedicated target. */
public class MultipleCommandSender {
	private AddressInterface aif;
	private Address recipient;
	private Vector commands_v;

	/** add a command to the bunch. 
	 * @param command String to add.
	*/
	public void add(String command) {
		Command c;
		try { c=Parser.parse(command); }
		catch (MBusParsingException mbuspe) {
			System.err.println("unparsable command to MultipleCommandSender.add: "+command);
			return;
		}
		commands_v.addElement(c);
	}

	/** reset this sender, means empty its command list. */
	public void removeAll() { commands_v.removeAllElements(); }

	/** trigger to send all added commands in one mbus message. */
	public void send() {
		int i;
		Command[] ca = new Command[commands_v.size()];
		for (i=0; i<commands_v.size(); i++) {
			ca[i]=(Command)commands_v.elementAt(i);
		}
		aif.send(ca,recipient);
	}

	/** @param sender MBUS address this message comes from.
	 *  @param recipient MBUS address this message goes to.
	 *  @param aif AddressInterface to use for sending.
	*/
	public MultipleCommandSender(Address recipient, AddressInterface aif) {
		this.recipient=recipient;
		commands_v=new Vector(5);
		this.aif=aif;
	}
}

