package bonephone;

import javax.swing.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;



// -------------------- berkomGUI ----------------------------------

/**
 * The berkomGUI class is an alternative look of the GUI class and is integrated upon 
 * insistation of berkom. It basically works the same way as the GUI class.
*/
public class berkomGUI implements PhoneBookListener, UINotifyable, ActionListener {

	final public static String versionstring="Version "+System.getProperty("bonephone.version")+" alpha";

	final public static int Style_Large  = 0;
	final public static int Style_Classic = 1;

	int style;

	JPanel            ButtonPane;
	JPanel            SubjectPane;
	JPanel            VolumePane;
	JPanel            ToPane;
	JTextField        SubjectField;   // Subject Text Field
	JTextField        ToField;        // To Text field for manual dialing
	IPT_State_GUI     state;
	JFrame            fr1;

	//private berkomPaneOpener        spo; // Subject pane
	private berkomPaneOpener        vpo; // volume pane
	private berkomPaneOpener        tpo; // to pane
	private berkomAudioControlPanel actl;
  
	private JButton        left,right;
	private JPanel         ButtonPanel;    // Holds the bottom line buttons 
	private JPanel         ListPanel;      // Holds the List and scrollbars
	private JScrollPane    ListScrollPane;  

	private PhoneController PC;
	private PhoneBook       PB;
	private Configuration   CFG;

	private PanelHolder    ph_calls;
	private PanelHolder    ph_pbook;
	//private PanelHolder    ph_splash;
	//private PanelHolder    ph_current; // PanelHolder which is currently displayed
	private JPanel    ph_splash;
	private JPanel    ph_current; // PanelHolder which is currently displayed
  
	private HashMap callPanelMap;      // Call -> CallPanel


	private ImageIcon iAudio0 = new ImageIcon("images.berkom/nav_unten_li.jpg");
	private ImageIcon iAudio = new ImageIcon("images.berkom/nav_unten_li_a.jpg");
	private ImageIcon iPhonebook0 = new ImageIcon("images.berkom/nav_unten_re.jpg");
	private ImageIcon iPhonebook = new ImageIcon("images.berkom/nav_unten_re_a.jpg");

	private ImageIcon iDial0 = new ImageIcon("images.berkom/nav_unten_annehmen.jpg");
	private ImageIcon iDial = new ImageIcon("images.berkom/nav_unten_annehmen_over.jpg");
	private ImageIcon iCancel0 = new ImageIcon("images.berkom/nav_unten_auflegen.jpg");
	private ImageIcon iCancel = new ImageIcon("images.berkom/nav_unten_auflegen_over.jpg");

	private ImageIcon iCtlOn0 = new ImageIcon("images.berkom/nav_unten_regler_on.jpg");
	private ImageIcon iCtlOn = new ImageIcon("images.berkom/nav_unten_regler_on_over.jpg");

	private ImageIcon iCtlOff0 = new ImageIcon("images.berkom/nav_unten_regler_off.jpg");
	private ImageIcon iCtlOff = new ImageIcon("images.berkom/nav_unten_regler_off_over.jpg");


	private void setPanelHolder(JPanel ph) {
		if (ph_current == ph) return;
		ListScrollPane.setViewportView(ph);
		ph_current=ph;
	}	

	// decide which panel and buttons to show
	private void sync() {
		switch (state.get()) {
			case IPT_State_GUI.ST_IDLE: {  
				//left.setEnabled(false);
				//left.setVisible(false);
				left.setEnabled(true);
				left.setVisible(true);
				// left.setText("Fake");
				left.setIcon(iAudio0);
				left.setRolloverIcon(iAudio);

				//right.setText("Phonebook");
				right.setIcon(iPhonebook0);
				right.setRolloverIcon(iPhonebook);
				//spo.close();
				tpo.close();
				vpo.close();
				setPanelHolder(ph_splash);
				break;
			}	
			case IPT_State_GUI.ST_PBOOK: {
				left.setEnabled(true);
				left.setVisible(true);
				//left.setText("Dial");
				//right.setText("Cancel");
				left.setIcon(iCancel0);
				left.setRolloverIcon(iCancel);
				right.setIcon(iDial0);
				right.setRolloverIcon(iDial);
				tpo.open(1);
				//spo.open(2);
				vpo.close();
				setPanelHolder(ph_pbook);
				fr1.getContentPane().validate();
				break;
			}
			case IPT_State_GUI.ST_CC: {
				left.setVisible(true);
				left.setEnabled(true);
				//left.setText("Audio");
				//right.setText("Phonebook");
				if (vpo.isOpen()) {
					left.setIcon(iCtlOff0);
					left.setRolloverIcon(iCtlOff);
				} else {
					left.setIcon(iCtlOn0);
					left.setRolloverIcon(iCtlOn);
				}
				right.setIcon(iPhonebook0);
				right.setRolloverIcon(iPhonebook);
				//spo.close();
				tpo.close();
				setPanelHolder(ph_calls);
				break;
			}
		}
	}


	private void handleLeftButton(String actioncommand) {
		if (state.get() == IPT_State_GUI.ST_IDLE) {
			System.out.println("Faking Call");
			PC.fakeCall();
			return;
		}
		if (state.get() == IPT_State_GUI.ST_PBOOK) { // CANCEL
			// check if Calls are present, then set state to ST_IDLE or ST_CC
			if (PC.getCalls().size()>0) {
				state.set(IPT_State_GUI.ST_CC);
			}
			else {
				state.set(IPT_State_GUI.ST_IDLE);
			}
			sync();
			return;
	  	}
		if (state.get() == IPT_State_GUI.ST_CC) {
			// open/close Volume Control Panel				
			if (vpo.isOpen()) {
				vpo.close();
			} else {
				vpo.open(1);
			}
			sync();
			return;
		}
	}


	private void handleRightButton (String actioncommand) {
		if (state.get() == IPT_State_GUI.ST_IDLE) {    // SPLASH
			state.set(IPT_State_GUI.ST_PBOOK);
			sync();
			return;
	  	}
		if (state.get() == IPT_State_GUI.ST_PBOOK) {
			PC.issueCall(ToField.getText(),SubjectField.getText());
			return;
		}
		if (state.get() == IPT_State_GUI.ST_CC) { // PHONEBOOK
			vpo.close();
			state.set(IPT_State_GUI.ST_PBOOK);
			sync();
			return;
		}
	}


	private CallPanel makeCallPanel(Call c) {
		CallPanel cp;
		boolean aa=CFG.getAsInt("AutoAnswer")!=0;
		switch (style) {
			case Style_Large:  { cp=new berkomLargeCallPanel(c,aa); break; }
			case Style_Classic: { cp=new ClassicCallPanel(c,aa); break; }
			default:            { cp=new ClassicCallPanel(c,aa); break; }
		}
		ph_calls.add(cp);
		callPanelMap.put(c,cp);
		return cp;
	}


	private void syncStyle() {
		VectorRead calls=PC.getCalls();
		int i;
		for (i=0; i<calls.size(); i++) {		
			Call c=(Call)(calls.elementAt(i));
			CallPanel cp=(CallPanel)(callPanelMap.get(c));
			ph_calls.remove(cp);
			cp.detach();
			callPanelMap.remove(c);
		}
		for (i=0; i<calls.size(); i++) {		
			Call c=(Call)(calls.elementAt(i));
			makeCallPanel(c);
		}
		switch (style) {
			case Style_Large:  { ph_calls.useGridLayout(); break; }	
			case Style_Classic: { ph_calls.useBoxLayout(); break; }	
		}
		// ph_calls.validate();
		ListPanel.validate();
	}
	
	///////////////////////////
	// ActionListener interface
	public void actionPerformed(ActionEvent e) {
		String actioncommand=e.getActionCommand();
		Object o=e.getSource();
		if (o instanceof JButton) {
			JButton src=(JButton)o;
			if (src == right) { handleRightButton(actioncommand); }
			if (src == left)  { handleLeftButton(actioncommand); }
		}
		if (o instanceof JCheckBoxMenuItem) {
			char c=actioncommand.charAt(0);
			switch (c) {
			case 'a': {
					CFG.set("AutoAnswer",""+((CFG.getAsInt("AutoAnswer")!=0)?0:1));
					break;
				}
			}
			return;
		}
		if (o instanceof JRadioButtonMenuItem) {
			char c=actioncommand.charAt(0);
			String p=actioncommand.substring(1);
			switch (c) {
			case 's': { // set style
					int sty;
					try { sty=Integer.parseInt(p); } catch (Exception ex) { break; }
					style=sty;
					CFG.set("CallPanelStyle",""+style);
					syncStyle();
					break;
				}
			case 'i': { // set input channel
					CFG.set("InputChannel",p);
					PC.setInputPort(p);
					break;
				}
			case 'o': { // set output channel
					CFG.set("OutputChannel",p);
					PC.setOutputPort(p);
					break;
				}
			case 'u': { // set upstream profile
					int i;
					CodecProfileManager CPM=PC.getCodecProfileManager();
					try { i=Integer.parseInt(p); } catch (Exception ex) { break; }
					CFG.set("UpstreamProfile",""+i);
					Vector pr=CPM.getProfile(i);
					CPM.setUpstreamProfile(pr);
					break;
				}
			case 'd': { // set upstream profile
					int i;
					CodecProfileManager CPM=PC.getCodecProfileManager();
					try { i=Integer.parseInt(p); } catch (Exception ex) { break; }
					CFG.set("DownstreamProfile",""+i);
					Vector pr=CPM.getProfile(i);
					CPM.setDownstreamProfile(pr);
					break;
				}
//			case 'v': { // set IP version to use (4 or 6)
//					if (p.equals("4")) main.useIPv(4);
//					if (p.equals("6")) main.useIPv(6);
//					break;
//				}
			}		
			return;
		}
	}


	//////////////////////////////
	// PhoneBookListener interface
	public void selectEntry (PhoneBookEntry pbe) {
		// Load URL to the ToField
		ToField.setText(pbe.getSIPURL());
		// SubjectField.setText("");
   	}
	public void useEntry (PhoneBookEntry pbe) {
		PC.issueCall(ToField.getText(),SubjectField.getText());
	}


	/////////////////////////
	// UINotifyable interface
	public void codecChanged (CodecDescriptor Codec) {}  // Re-Display current codec
	public void newCall (Call c) {    
		makeCallPanel(c);
		state.set(IPT_State_GUI.ST_CC);
		sync();
	}
	public void deletedCall (Call c) {
		CallPanel cp=(CallPanel)(callPanelMap.get(c));
		ph_calls.remove(cp);
		int n=PC.getCalls().size();
		int s=state.get();
		if (n==0 && s==IPT_State_GUI.ST_CC) { state.set(IPT_State_GUI.ST_IDLE); }
		sync();
	}


	//////////////
	// Constructor
	public berkomGUI (PhoneController PC, PhoneBook PB, Configuration CFG) {

		callPanelMap=new HashMap();
		//Dimension d=new Dimension(310,220);
		//Dimension d=new Dimension(400,500);
		Dimension d=new Dimension(400,540);
		int i;    

		this.PC=PC;
		this.PB=PB;
		this.CFG=CFG;

		CFG.setUnset("CallPanelStyle",""+Style_Classic);
		style=CFG.getAsInt("CallPanelStyle");

	    state=new IPT_State_GUI(IPT_State_GUI.ST_IDLE);
		fr1 = new JFrame("Bone Phone "+versionstring);

		//ph_splash=new PanelHolder(d);

		//Vector splash=new Vector(5);
		//splash.add(" "+CFG.getAsString("LogoLabel"));
		//splash.add(new ImageIcon(CFG.getAsString("LogoImage")));
		//splash.add(" ");
		//splash.add(new ImageIcon("images/BonePhoneLogo.gif"));
		//splash.add(versionstring);
		//splash.add(" ");
		//ph_splash.add(new SplashPanel(splash));

		Vector devs=PC.getAudioDevices();
		//for (i=0; i<devs.size(); i++) {
		//	splash.add("Device: "+(String)(devs.elementAt(i)));
		//}
		//splash.add(" ");

		JLabel il;
		ph_splash=new JPanel();
		ph_splash.setMinimumSize(d);
		ph_splash.setLayout(new BoxLayout(ph_splash, BoxLayout.Y_AXIS));
		ph_splash.setBackground( new Color(7,175,193));
		ph_splash.setBorder(null);

		il = new JLabel(new ImageIcon("images.berkom/introscreen.jpg"));
		il.setLayout(new BoxLayout(il, BoxLayout.Y_AXIS));
		il.setAlignmentX(Container.CENTER_ALIGNMENT);
		ph_splash.add(il);

		ph_pbook =new PanelHolder(d);
		ph_calls =new PanelHolder(d);

		ph_pbook.add(new berkomPhoneBookPanel(PB,this));

		try {
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		} catch (Exception exc) { 
			System.err.println ("Can't set Look&Feel, aborting");
			System.exit(-1);
		}

		ButtonPanel= new JPanel();
		ButtonPanel.setLayout(new BoxLayout(ButtonPanel, BoxLayout.X_AXIS));
		JLabel buttonLabel = new JLabel(new ImageIcon("images.berkom/nav_unten.jpg"));
		buttonLabel.setLayout(new BorderLayout(0,0));
		ButtonPanel.add(buttonLabel);

		//left  = new JButton("Left");
		//right = new JButton("Right");
		left  = new JButton();
		right = new JButton();
		left.addActionListener(this);
		right.addActionListener(this);

		Insets ix=new Insets(0,0,0,0);
		left.setMargin(ix);
		right.setMargin(ix);
		left.setBorder(null);
		right.setBorder(null);

		//buttonLabel.add(left);
		//buttonLabel.add(Box.createHorizontalGlue());
		//buttonLabel.add(right);

		buttonLabel.add(left, BorderLayout.WEST);
		buttonLabel.add(Box.createHorizontalGlue(), BorderLayout.CENTER);
		buttonLabel.add(right, BorderLayout.EAST);
    
		ListScrollPane=new JScrollPane(ph_splash);
		ListScrollPane.setBorder(null);
		//ListScrollPane.setMinimumSize(d);
		// ListScrollPane.setMaximumSize(d);
		//ListScrollPane.setPreferredSize(d);
		ListPanel=new JPanel();
		//ListPanel.setBackground(Color.white);
		ListPanel.setBackground(new Color(32,68,164));
		//ListPanel.setLayout(new BoxLayout(ListPanel, BoxLayout.X_AXIS));
		ListPanel.setLayout(new GridLayout(0,1));
		ListPanel.add(ListScrollPane);
    
	if (false) {
		JPanel dialBox=new JPanel();
		// top, left, bottom, right
		dialBox.setBorder(BorderFactory.createEmptyBorder(5,20,5,20));
		dialBox.setLayout(new BorderLayout(15,5));
		dialBox.setBackground(new Color(32,68,164));

		JPanel dialPane=new JPanel();
		dialPane.setBackground( new Color(32,68,164));

		dialPane.setLayout(new GridLayout(2,2,10,10));
		dialBox.add(dialPane, BorderLayout.CENTER);

		JLabel cl = new JLabel("Call:");
		cl.setHorizontalTextPosition(JLabel.RIGHT);
		dialPane.add(cl);
		ToField=new JTextField();
		ToField.setBorder(null);
		ToField.setMargin(new Insets(5,5,5,5));
		dialPane.add(ToField);

		JLabel sl = new JLabel("Subject:");
		sl.setHorizontalTextPosition(JLabel.RIGHT);
		dialPane.add(sl);
		SubjectField=new JTextField();
		SubjectField.setBorder(null);
		SubjectField.setMargin(new Insets(5,5,5,5));
		// SubjectField.addActionListener(guic.displ);
		dialPane.add(SubjectField);

		tpo=new berkomPaneOpener(fr1.getContentPane(),300,dialBox);
	} else {
		JPanel dialBox=new JPanel();
		dialBox.setBackground( new Color(32,68,164));

		JLabel dialLabel = new JLabel(new ImageIcon("images.berkom/textfeld.jpg"));
		dialLabel.setLayout(new BoxLayout(dialLabel, BoxLayout.Y_AXIS));
		dialBox.add(dialLabel);

		JPanel dialPane=new JPanel();
		// top, left, bottom, right
		dialPane.setBorder(BorderFactory.createEmptyBorder(10,90,10,60));
		//dialPane.setLayout(new BoxLayout(dialPane, BoxLayout.Y_AXIS));
		dialPane.setLayout(new GridLayout(2,1,10,10));
		dialPane.setOpaque(false);

		ToField=new JTextField();
		ToField.setBorder(null);
		//ToField.setBorder(BorderFactory.createEmptyBorder(0,0,1,0));
		//ToField.setOpaque(false);
		dialPane.add(ToField);

		SubjectField=new JTextField();
		SubjectField.setBorder(null);
		//SubjectField.setOpaque(false);
		dialPane.add(SubjectField);

		dialLabel.add(dialPane);
		tpo=new berkomPaneOpener(fr1.getContentPane(),300,dialBox);
	}


		actl=new berkomAudioControlPanel(PC,CFG);
		// actl.setBorder(BorderFactory.createEmptyBorder(5,15,0,15));
		vpo=new berkomPaneOpener(fr1.getContentPane(),300,actl);
    
		Container ContentPane;
		ContentPane=fr1.getContentPane();
		ContentPane.setLayout(new BoxLayout(ContentPane, BoxLayout.Y_AXIS));
		ContentPane.add(ListPanel);
		ContentPane.add(ButtonPanel);
		fr1.addWindowListener(new WindowAdapter () {
			public void windowClosing(WindowEvent e) {
				try { berkomGUI.this.CFG.writeOut(); } catch (Exception ex) {}
				System.out.println("Tschuess !!!");
				System.exit(0);
			}
		}
		);

		JMenuBar menuBar;
		JMenu menu;
		JMenuItem menuItem;
		JRadioButtonMenuItem rb1,rb2,rb;
		menuBar = new JMenuBar();
		fr1.setJMenuBar(menuBar);

		menu = new JMenu("Connectivity");
		menu.setMnemonic(KeyEvent.VK_R);
		menu.getAccessibleContext().setAccessibleDescription("Set up/down codec profiles");
		menuBar.add(menu);

		JMenu upmenu = new JMenu("Upstream");
		JMenu downmenu = new JMenu("Downstream");
		ButtonGroup upgroup = new ButtonGroup();
		ButtonGroup downgroup = new ButtonGroup();
		ButtonGroup group = new ButtonGroup();
		CodecProfileManager CPM=PC.getCodecProfileManager();
		for (i=0; i<CPM.getNumberOfProfiles(); i++) {
			rb1 = new JRadioButtonMenuItem(CPM.getDisplayName(i));
			rb1.setActionCommand("u"+i);
			rb1.addActionListener(this);
			if (CFG.getAsInt("UpstreamProfile")==i) rb1.setSelected(true);
			rb2 = new JRadioButtonMenuItem(CPM.getDisplayName(i));
			rb2.setActionCommand("d"+i);
			rb2.addActionListener(this);
			if (CFG.getAsInt("DownstreamProfile")==i) rb2.setSelected(true);
			upgroup.add(rb1);
			upmenu.add(rb1);
			downgroup.add(rb2);
			downmenu.add(rb2);
		}
		menu.add(upmenu);
		menu.add(downmenu);
		menu.addSeparator();

		JCheckBoxMenuItem cbmi=new JCheckBoxMenuItem("Auto-answer");
		cbmi.setActionCommand("a");
		cbmi.addActionListener(this);
		cbmi.setSelected(CFG.getAsInt("AutoAnswer")!=0);
		menu.add(cbmi);

//		rb1 = new JRadioButtonMenuItem("use IPv4");
//		rb1.setActionCommand("v4");
//		rb1.addActionListener(this);
//		menu.add(rb1);
//		group.add(rb1);
//		rb2 = new JRadioButtonMenuItem("use IPv6");
//		rb2.setActionCommand("v6");
//		rb2.addActionListener(this);
//		menu.add(rb2);
//		group.add(rb2);
//		int ipv=CFG.getAsInt("IPVersion");
//		if (main.checkIPv6()) {
//			if (ipv==4)	rb1.setSelected(true);  
//			if (ipv==6)	rb2.setSelected(true);  
//		} else {
//			rb1.setSelected(true);
//			// rb2.setEnabled(false);
//		}
	
		menu = new JMenu("Ports");
		menu.setMnemonic(KeyEvent.VK_P);
		menu.getAccessibleContext().setAccessibleDescription("Set the audio in-/output port to use");
		menuBar.add(menu);
		group= new ButtonGroup();

		Vector p;  String s;
		p=PC.getOutputPorts();
		for (i=0; i<p.size(); i++) {
			s=(String)(p.elementAt(i));
			rb = new JRadioButtonMenuItem(s);
			rb.setActionCommand("o"+s);
			rb.addActionListener(this);
			if (CFG.getAsString("OutputChannel").equals(s)) {
				rb.setSelected(true);
			}
			group.add(rb);
			menu.add(rb);
		}

		menu.addSeparator();
		group=new ButtonGroup();

		p=PC.getInputPorts();
		for (i=0; i<p.size(); i++) {
			s=(String)(p.elementAt(i));
			rb = new JRadioButtonMenuItem(s);
			rb.setActionCommand("i"+s);
			rb.addActionListener(this);
			if (CFG.getAsString("InputChannel").equals(s)) {
				rb.setSelected(true);
			}
			group.add(rb);
			menu.add(rb);
		}

		menu = new JMenu("Style");
		menu.setMnemonic(KeyEvent.VK_S);
		menu.getAccessibleContext().setAccessibleDescription("Set the call rendering style");
		menuBar.add(menu);
		group=new ButtonGroup();
		JRadioButtonMenuItem mi;
		mi=new JRadioButtonMenuItem("Classic");
		mi.addActionListener(this);
		mi.setActionCommand("s"+Style_Classic);
		if (style==Style_Classic) mi.setSelected(true);
		group.add(mi);
		menu.add(mi);
		mi=new JRadioButtonMenuItem("Large");
		mi.addActionListener(this);
		mi.setActionCommand("s"+Style_Large);
		if (style==Style_Large) mi.setSelected(true);
		group.add(mi);
		menu.add(mi);

		sync();
		syncStyle();

		fr1.pack();
		fr1.setVisible(true);

		PC.addPhoneObserver(this);    
	} 

}  // class

